#!/usr/bin/perl -w
#
# Author: Andrew Robert
# Date:   06/13/2005
# Program: push_messages.pl

use strict;
use MQSeries;
use MQSeries::Queue;
use MQSeries::Message;
use Getopt::Std;


#
# General level variables
#
my $count=0;
my $status="";
my $qmgr="";
my $targetq="";
my $val="";
my $key=""; 
my $infile=""; 
my $line="";

#my ( $line, $send_message, $putmsg, $message, $numkeys, $key, $val )   = ""; 
my %message_hash;

#
# Read and assign input from command line 
#
my %opts;
getopts("m:q:f:", \%opts);

                    


if  ( keys(%opts) != 3 )    
    {
     #
     # Test number of command line arguments. Exit and 
     # supply usage summary if not enough were entered.
     #     
     print "\nMissing command line parameters\n";
     print "Usage:   push_messages.pl -m {queue manager} -q {queue} -f {input file}\n\n"; 
     exit;
    }
else
   {
    #     
    # If enough arguments were supplied, capitilize the queue and queue manager  
    # in case they were entered in lower case.  
    #   
    $targetq   = uc( $opts{q} );  
    $qmgr      = uc( $opts{m} );
    $infile    = lc( $opts{f} );
   }


# Create source queue object 
# linking queue manager and destination queue
#
my $targetqueue = MQSeries::Queue->new
    (
     QueueManager       => $qmgr,
     Queue              => $targetq,
     Mode               => 'output'
    )|| die ("Unable to instantiate $qmgr::$targetq object\n");


#
# Check if input file exists. If it doesn't, display a warning and exit.
#    
if (not -e $infile )
   {
    die ("\nFile $infile does not exist.. Please check the file name and try again\n\n");
   }  



open( INFILE, "<", $infile )  or die("Couldn't open $infile : $!\n") ;  


#
# Walk through each line of the input file until end of file flag is detected.
#
while ( <INFILE> )
            {
             my $line=$_;

             #
             # Skip if blank lines are detected in input file
             #
             if  ( $line =~/^\s*$/  )
                 {
                  next;
                 }
             #
             # Remove artificially inserted new-line character
             #
             chomp $line;
             
             if ( $line eq "<STARTMESSAGE>" )
                {
                 $count++;
                }
             elsif ( $line eq "<ENDMESSAGE>" )
                {
                 my $putmsg = &makemessage( %message_hash );
                 $targetqueue->Put( Message => $putmsg );
                 %message_hash=();
                }
              else
                {
                 #
                 # Decode line
                 #
                 $line =~s/(\\\\)|\\n/$1?"\\":"\n"/eg;

                 #
                 # Break the line to a key value pair
                 #
                 ($key, $val) = split ( /:/, $line, 2 );

                 #
                 # Add the data to a hash
                 #
                 $message_hash{$key} = $val;
                }
            } 



close (INFILE);
print "\nTotal Messages Processed = $count\n";



sub makemessage
    {
     my %param = @_;  
     
     my $putmsg = MQSeries::Message->new
                        (
                         MsgDesc => 
                               {
                                PutTime          => $param{PutTime},
                                ReplyToQ         => $param{ReplyToQ}, 
                                GroupId          => $param{GroupId},
                                UserIdentifier   => $param{UserIdentifier},
                                Encoding         => $param{Encoding},
                                ApplIdentityData => $param{ApplIdentityData},
                                BackoutCount     => $param{BackoutCount},
                                Format           => $param{Format},
                                MsgFlags         => $param{MsgFlags},
                                ApplOriginData   => $param{ApplOriginData},
                                PutApplName      => $param{PutAppName},
                                MsgType          => $param{MsgType},
                                MsgSeqNumber     => $param{MsgSeqNumber},
                                CodedCharSetId   => $param{CodedCharSetId},
                                AccountingToken  => $param{AccountingToken},
                                Offset           => $param{Offset},
                                CorrelId         => $param{CorrelId},
                                ReplyToQMgr      => $param{ReplyToQMgr},
                                Persistence      => $param{Persistence},
                                PutDate          => $param{PutDate},
                                OriginalLength   => $param{OriginalLength},
                                PutApplType      => $param{PutApplType},
                                Expiry           => $param{Expiry},
                                Version          => $param{Version},
                                Priority         => $param{Priority},
                                Feedback         => $param{Feedback},
                                Report           => $param{Report},
                                MsgId            => $param{MsgId}
                               },
                         Data         => $param{Data}
                        ); 
     return $putmsg;
    }

